''************************************************
''*  StepperMotor Drive                          *
''*  Original Author: Jim Miller                 *
''*  Copyright (c) 2014 Cannibal Robotics, Inc.  *
''*  See end of file for terms of use.           *
''************************************************

CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  SerialOutput  = 9
  ClockInput    = 8
  SwitchInBase  = 20

  Xstep         = 11            'D2
  Ystep         = 10            'D3
  Zstep         = 7             'D4
  Xdir          = 6             'D5
  Ydir          = 5             'D6
  Zdir          = 4             'D7
            
  Enable        = 12

  Xmarker       = 19
  Ymarker       = 18

Var
  long  Stack[200]
  long  Xcount,Ycount,Zcount
  long  Xtarget,Ytarget,Ztarget                                    
  long  Xrev,Yrev,Zrev   
  long  Xdelay,Ydelay,Zdelay
  
  byte  Lock

Pub Init(StepLock)
  cognew(Main(StepLock), @Stack)    
  return @Xcount
  
Pub Main(StepLock)
  Lock := StepLock
  SetUp
  Xdelay := 35_000
  Ydelay := 30_000
  Zdelay := 20_000

  Repeat 3
    FindXMarker
  MoveDegreesX(18)    
  Xcount := 0
  Xtarget := 0  
  
  Repeat 4
    FindYMarker    
  MoveDegreesY(-170)  
  Ycount := 0
  Ytarget := 0

  Repeat     

Pub FindXMarker
  Repeat while ina[Xmarker] == 0
    stepX(1)      

  Repeat while ina[Xmarker] == 1
    stepX(1)
    
  Xrev := Xcount 
  Xcount := 0

Pub FindYMarker
  Repeat while ina[Ymarker] == 0
    stepY(1)      

  Repeat while ina[Ymarker] == 1
    stepY(1)
    
  Yrev := Ycount 
  Ycount := 0   

pub MoveDegreesX(d)
  d := (d * Xrev) / 360
  StepX(d)

pub MoveDegreesY(d)
  d := (d * Yrev) / 360
  StepY(d)
pub EnableMotor(s)

  dira[Enable] ~~
  outa[Enable] := s
pub Stx
  StepX(1)
pub Sty
  StepY(1)
     
Pub StepX(d) | i  ' Steps Motor X d times (- == reverse)

  if d < 0                      ' Test for direction and set output
    OutA[Xdir] := 1
    i := -1                     ' Set step count increment -
  else
    OutA[Xdir] := 0 
    i := 1                      ' Set step count increment +
    
  OutA[Enable] := 0             ' Enable the stepper controller
  
  Repeat ||d                    ' Repeat d steps ( || means absolute value for negitive directions)
    OutA[Xstep] := 1            ' High on stepper controller
    Waitcnt(cnt + Xdelay)       ' Wait Xdelay
    OutA[Xstep] := 0            ' Low on stepper controller
    Waitcnt(cnt + Xdelay)       ' Wait Xdelay again
    
    lockset(Lock)               ' Lock set to prevent external read conflict
    XCount := Xcount + i        ' Bump the step count by increment
    LockClr(Lock)               ' clear the lock for reads
  
  OutA[Enable] := 1             ' disable the controller
  Return Xcount                 ' Send Xcount back

Pub StepY(d) | i ' Steps Motor Y d times (- == reverse)

  if d < 0
    OutA[Ydir] := 1
    i := -1
  else
    OutA[Ydir] := 0
    i := 1
  OutA[Enable] := 0
  Repeat ||d
    OutA[Ystep] := 1      
    Waitcnt(cnt + Ydelay)
    OutA[Ystep] := 0  
    Waitcnt(cnt + Ydelay) 
    lockset(Lock)
    YCount := Ycount + i   
    LockClr(Lock)   
  OutA[Enable] := 1 
  Return Ycount

Pub StepZ(d) | i ' Steps Motor Z d times (- == reverse)
  if d < 0
    OutA[Zdir] := 1
    i := -1
  else
    OutA[Zdir] := 0
    i := 1                  
  Repeat ||d
    OutA[Zstep] := 1      
    Waitcnt(cnt + Zdelay)  
    OutA[Zstep] := 0  
    Waitcnt(cnt + Zdelay)  
    lockset(Lock)
    ZCount := Zcount + i 
    LockClr(Lock) 
  Return Zcount

Pri SetUp     '' Setup Ports for operation
  
  dira[Xstep] ~~   
  dira[Ystep] ~~ 
  dira[Zstep] ~~ 
  dira[Xdir]  ~~ 
  dira[Ydir]  ~~ 
  dira[Zdir]  ~~
  dira[Enable] ~~
  
  outa[Xstep] := 0   
  outa[Ystep] := 0 
  outa[Zstep] := 0 
  outa[Xdir]  := 0 
  outa[Ydir]  := 0 
  outa[Zdir]  := 0    
  outa[Enable] := 0